@@ -52,7 +52,7 @@ namespace nsCore
         CWE17CLIModule& operator = (const CWE17CLIModule&);
 
         nsWireless::CWE17Driver m_Driver;
-        nsUserData::CProfilesFile<> m_ProfilesFile;
+        nsUserData::CProfilesFile m_ProfilesFile;
 
     }; // CWE17CLIModule
 
@@ -65,23 +65,24 @@ namespace nsCore
         ~CWE17Module () throw();
         nsWireless::IDriver& GetDriver () throw();
         const nsWireless::IDriver& GetDriver () const throw();
-        nsUserData::CProfile* NewProfile () throw();
-        nsUserData::CProfile* NewProfile (const nsWireless::CCell&) throw();
-        bool EditProfile (unsigned) throw();
+        nsUserData::CProfile* NewProfile (::GtkWindow*) throw();
+        nsUserData::CProfile* NewProfile
+                              (::GtkWindow*, const nsWireless::CCell&) throw();
+        bool EditProfile (::GtkWindow*, unsigned) throw();
         void DeleteProfile (unsigned) throw();
         void ApplyProfile (unsigned) throw (nsErrors::CSystemExc);
+        void ApplyProfile (unsigned, const nsWireless::CMacAddress&)
+                                                throw (nsErrors::CSystemExc);
         /** \warning No bound check! */
         nsUserData::CProfile& GetProfile (unsigned) throw();
         unsigned GetNbProfile () const throw();
         void RecordProfiles () throw (nsErrors::CException);
-        void Connect (const nsWireless::CCell&)
-                                throw (nsErrors::CSystemExc, std::bad_alloc);
         ::GtkWidget* GetPrivateTab () throw();
         void BecomeRoot () throw (nsErrors::CException, std::bad_alloc);
 
 
       protected :
-        nsUserData::CProfile* EditNewProfile () throw();
+        nsUserData::CProfile* EditNewProfile (::GtkWindow*) throw();
         static const char* const NewProfileDefaultName;
 
 
@@ -91,7 +92,7 @@ namespace nsCore
 
         std::auto_ptr<nsWireless::CWE17Driver> m_pDriver;
         nsCore::CProfileEditor m_Editor;
-        nsUserData::CProfilesFile<> m_ProfilesFile;
+        nsUserData::CProfilesFile m_ProfilesFile;
 
     }; // CWE17Module
 
