@@ -44,7 +44,8 @@ namespace nsErrors
                        UnknownExc = -23, NoDriverSet = -24,
                        NoHandlerDefined = -25, InvalidRootPassword = -26,
                        InvalidDataFromHelper = -27, UnexpectedStreamEnd = -28,
-                       RT2400AdHocLimitation = -29, StatsParsing = -30,
+                            // FIXME Error -29 available
+                       StatsParsing = -30,
                        RT2570EncIoctlNotFound = -31,
                        RT2570AuthIoctlNotFound = -32,
                        RT2570KeyIoctlNotFound = -33, InvalidInstanceNum = -34,
@@ -53,7 +54,8 @@ namespace nsErrors
                        OptionsRecordFailure = -37, SaltBufferOverflow = - 38,
                        HelperNotRoot = -39, RutilTAuthenticationFailed = -40,
                        DataParsingError1 = -41, DataParsingError2 = -42,
-                       CannotSendRootPasswd = -43, CannotExecuteHelper = -44};
+                       DataParsingError3 = -43, CannotSendRootPasswd = -44,
+                       CannotExecuteHelper = -45};
 
 } // namespace nsErrors
 
