@@ -25,8 +25,7 @@
 
 namespace nsNet
 {
-        // NextC is designed to extend the protocol.
-    enum IfaceProtocolCodes_e {DeviceNameC, DhcpC, StaticIpC, NextC};
+    enum IfaceProtocolCodes_e {DeviceNameC, DhcpC, StaticIpC, UpC, DownC};
  
     const char* const IfaceHandler ("IfaceHandler");
 
