@@ -36,6 +36,8 @@ namespace nsNet
         CInterfaceRoot (const CInterface&) throw (nsErrors::CSystemExc);
         void SetIP (const CIPAddress&) throw (nsErrors::CSystemExc);
         void SetIPFromDHCP (const std::string&) throw (nsErrors::CException);
+        void Up () throw (nsErrors::CSystemExc);
+        void Down () throw (nsErrors::CSystemExc);
 
 
       protected :
