@@ -38,6 +38,7 @@ namespace nsWireless
     class IDriver;
     class CCell;
     class CDeviceInfo;
+    class CMacAddress;
 
 } // namespace nsWireless
 
@@ -73,17 +74,17 @@ namespace nsCore
         virtual ~IModule () throw() {}
         virtual nsWireless::IDriver& GetDriver () throw() = 0;
         virtual const nsWireless::IDriver& GetDriver () const throw() = 0;
-        virtual nsUserData::CProfile* NewProfile () throw() = 0;
-        virtual nsUserData::CProfile* NewProfile (const nsWireless::CCell&)
-                                                                throw() = 0;
-        virtual bool EditProfile (unsigned) throw() = 0;
+        virtual nsUserData::CProfile* NewProfile (::GtkWindow*) throw() = 0;
+        virtual nsUserData::CProfile* NewProfile
+                          (::GtkWindow*, const nsWireless::CCell&) throw() = 0;
+        virtual bool EditProfile (::GtkWindow*, unsigned) throw() = 0;
         virtual void DeleteProfile (unsigned) throw() = 0;
         virtual void ApplyProfile (unsigned) throw (nsErrors::CSystemExc) = 0;
+        virtual void ApplyProfile (unsigned, const nsWireless::CMacAddress&)
+                                            throw (nsErrors::CSystemExc) = 0;
         virtual nsUserData::CProfile& GetProfile (unsigned) throw() = 0;
         virtual unsigned GetNbProfile () const throw() = 0;
         virtual void RecordProfiles () throw (nsErrors::CException) = 0;
-        virtual void Connect (const nsWireless::CCell&)
-                            throw (nsErrors::CSystemExc, std::bad_alloc) = 0;
         /** \return A GUI component to set driver specific parameters (may
          *          return 0 if none). */
         virtual ::GtkWidget* GetPrivateTab () throw() = 0;
