@@ -121,6 +121,7 @@ namespace nsWireless
         void SetFromASCII (const std::string&) throw (std::bad_alloc);
 
         bool operator == (const CHexaKey&) const throw();
+        bool operator != (const CHexaKey&) const throw();
 
 
       private :
@@ -166,7 +167,7 @@ namespace nsWireless
     }; // CMacAddress
  
 
-    enum EncryptType_e {None, WEP, AES, TKIP, UnknownEnc};
+    enum EncryptType_e {None, WEP, AES, TKIP, TKIPAES, UnknownEnc};
     enum AuthType_e {Open, Shared, WPAPSK, WPANONE, WPA2PSK, WPA, WPA2,
                      UnknownAuth};
 
