@@ -23,6 +23,8 @@
 
 #include <memory> // auto_ptr<>
 
+#include <gtk/gtkwindow.h>
+
 
 
 namespace nsUserData {class CProfile;}
@@ -44,7 +46,7 @@ namespace nsCore
         ~CProfileEditor () throw();
         nsGUI::CConfigDialog* GetDialog () throw();
         const nsGUI::CConfigDialog* GetDialog () const throw();
-        bool Edit (nsUserData::CProfile&) throw();
+        bool Edit (::GtkWindow*, nsUserData::CProfile&) throw();
 
 
       private :
