@@ -88,7 +88,7 @@ namespace nsCore
         CRT2400CLIModule& operator = (const CRT2400CLIModule&);
 
         nsWireless::CRT2400Driver m_Driver;
-        nsUserData::CProfilesFile<> m_ProfilesFile;
+        nsUserData::CProfilesFile m_ProfilesFile;
 
     }; // CRT2400CLIModule
 
@@ -102,16 +102,17 @@ namespace nsCore
         void BecomeRoot () throw (nsErrors::CException, std::bad_alloc);
         nsWireless::IDriver& GetDriver () throw();
         const nsWireless::IDriver& GetDriver () const throw();
-        nsUserData::CProfile* NewProfile () throw();
-        nsUserData::CProfile* NewProfile (const nsWireless::CCell&) throw();
-        bool EditProfile (unsigned) throw();
+        nsUserData::CProfile* NewProfile (::GtkWindow*) throw();
+        nsUserData::CProfile* NewProfile
+                              (::GtkWindow*, const nsWireless::CCell&) throw();
+        bool EditProfile (::GtkWindow*, unsigned) throw();
         void DeleteProfile (unsigned) throw();
         void ApplyProfile (unsigned) throw (nsErrors::CSystemExc);
+        void ApplyProfile (unsigned, const nsWireless::CMacAddress&)
+                                                throw (nsErrors::CSystemExc);
         nsUserData::CProfile& GetProfile (unsigned) throw();
         unsigned GetNbProfile () const throw();
         void RecordProfiles () throw (nsErrors::CException);
-        void Connect (const nsWireless::CCell&)
-                                throw (nsErrors::CSystemExc, std::bad_alloc);
         ::GtkWidget* GetPrivateTab () throw();
 
 
@@ -123,11 +124,11 @@ namespace nsCore
         CRT2400Module (const CRT2400Module&);
         CRT2400Module& operator= (const CRT2400Module&);
 
-        nsUserData::CProfile* EditNewProfile () throw();
+        nsUserData::CProfile* EditNewProfile (::GtkWindow*) throw();
 
         std::auto_ptr<nsWireless::CRT2400Driver> m_pDriver;
         CProfileEditor m_ProfileEditor;
-        nsUserData::CProfilesFile<> m_ProfilesFile;
+        nsUserData::CProfilesFile m_ProfilesFile;
 
     }; // CRT2400Module
 
@@ -147,8 +148,8 @@ namespace nsCore
         CRTAdvancedCLIModule (nsWireless::CRTAdvancedDriver*,
                               const std::string&)
                                 throw (nsErrors::CException, std::bad_alloc);
-        nsUserData::CProfilesFile<>& GetProfilesFile () throw();
-        const nsUserData::CProfilesFile<>& GetProfilesFile () const throw();
+        nsUserData::CProfilesFile& GetProfilesFile () throw();
+        const nsUserData::CProfilesFile& GetProfilesFile () const throw();
 
 
       private :
@@ -156,7 +157,7 @@ namespace nsCore
         CRTAdvancedCLIModule& operator = (const CRTAdvancedCLIModule&);
 
         std::auto_ptr<nsWireless::CRTAdvancedDriver> m_pDriver;
-        nsUserData::CProfilesFile<> m_ProfilesFile;
+        nsUserData::CProfilesFile m_ProfilesFile;
 
     }; // CRTAdvancedCLIModule
 
@@ -167,16 +168,17 @@ namespace nsCore
         ~CRTAdvancedModule () throw();
         nsWireless::IDriver& GetDriver () throw();
         const nsWireless::IDriver& GetDriver () const throw();
-        nsUserData::CProfile* NewProfile () throw();
-        nsUserData::CProfile* NewProfile (const nsWireless::CCell&) throw();
-        bool EditProfile (unsigned) throw();
+        nsUserData::CProfile* NewProfile (::GtkWindow*) throw();
+        nsUserData::CProfile* NewProfile
+                              (::GtkWindow*, const nsWireless::CCell&) throw();
+        bool EditProfile (::GtkWindow*, unsigned) throw();
         void DeleteProfile (unsigned) throw();
         void ApplyProfile (unsigned) throw (nsErrors::CSystemExc);
+        void ApplyProfile (unsigned, const nsWireless::CMacAddress&)
+                                                throw (nsErrors::CSystemExc);
         nsUserData::CProfile& GetProfile (unsigned) throw();
         unsigned GetNbProfile () const throw();
         void RecordProfiles () throw (nsErrors::CException);
-        void Connect (const nsWireless::CCell&)
-                                throw (nsErrors::CSystemExc, std::bad_alloc);
         ::GtkWidget* GetPrivateTab () throw();
 
 
@@ -192,11 +194,12 @@ namespace nsCore
                                                                     throw();
         void SetDriver (nsWireless::CRTAdvancedDriver*) throw();
 
-        nsUserData::CProfilesFile<>& GetProfilesFile () throw();
-        const nsUserData::CProfilesFile<>& GetProfilesFile () const throw();
+        nsUserData::CProfilesFile& GetProfilesFile () throw();
+        const nsUserData::CProfilesFile& GetProfilesFile () const throw();
         static const char* const NewProfileDefaultName;
         CProfileEditor& GetProfileEditor () throw();
         const CProfileEditor& GetProfileEditor () const throw();
+        nsUserData::CProfile* EditNewProfile (::GtkWindow*) throw();
 
 
       private :
@@ -205,9 +208,7 @@ namespace nsCore
 
         std::auto_ptr<nsWireless::CRTAdvancedDriver> m_pDriver;
         CProfileEditor m_ProfileEditor;
-        nsUserData::CProfilesFile<> m_ProfilesFile;
-
-        nsUserData::CProfile* EditNewProfile () throw();
+        nsUserData::CProfilesFile m_ProfilesFile;
 
             // Private tab :
         ::GtkTable* const m_pTable;
@@ -258,6 +259,8 @@ namespace nsCore
         CRT2500Module (const std::string&, const std::string&, nsGUI::CGtkGUI*)
                                 throw (nsErrors::CException, std::bad_alloc);
         void BecomeRoot () throw (nsErrors::CException, std::bad_alloc);
+        nsUserData::CProfile* NewProfile
+                              (::GtkWindow*, const nsWireless::CCell&) throw();
 
     }; // CRT2500Module
 
@@ -318,7 +321,7 @@ namespace nsCore
         CRT2570CLIModule& operator = (const CRT2570CLIModule&);
 
         nsWireless::CRT2570Driver m_Driver;
-        nsUserData::CProfilesFile<> m_ProfilesFile;
+        nsUserData::CProfilesFile m_ProfilesFile;
 
     }; // CRT2570CLIModule
 
@@ -332,16 +335,17 @@ namespace nsCore
         void BecomeRoot () throw (nsErrors::CException, std::bad_alloc);
         nsWireless::IDriver& GetDriver () throw();
         const nsWireless::IDriver& GetDriver () const throw();
-        nsUserData::CProfile* NewProfile () throw();
-        nsUserData::CProfile* NewProfile (const nsWireless::CCell&) throw();
-        bool EditProfile (unsigned) throw();
+        nsUserData::CProfile* NewProfile (::GtkWindow*) throw();
+        nsUserData::CProfile* NewProfile
+                              (::GtkWindow*, const nsWireless::CCell&) throw();
+        bool EditProfile (::GtkWindow*, unsigned) throw();
         void DeleteProfile (unsigned) throw();
         void ApplyProfile (unsigned) throw (nsErrors::CSystemExc);
+        void ApplyProfile (unsigned, const nsWireless::CMacAddress&)
+                                                throw (nsErrors::CSystemExc);
         nsUserData::CProfile& GetProfile (unsigned) throw();
         unsigned GetNbProfile () const throw();
         void RecordProfiles () throw (nsErrors::CException);
-        void Connect (const nsWireless::CCell&)
-                                throw (nsErrors::CSystemExc, std::bad_alloc);
         ::GtkWidget* GetPrivateTab () throw();
 
 
@@ -354,11 +358,11 @@ namespace nsCore
         CRT2570Module (const CRT2570Module&);
         CRT2570Module& operator= (const CRT2570Module&);
 
-        nsUserData::CProfile* EditNewProfile () throw();
+        nsUserData::CProfile* EditNewProfile (::GtkWindow*) throw();
 
         std::auto_ptr<nsWireless::CRT2570Driver> m_pDriver;
         CProfileEditor m_ProfileEditor;
-        nsUserData::CProfilesFile<> m_ProfilesFile;
+        nsUserData::CProfilesFile m_ProfilesFile;
         nsGUI::CGtkGUI* const m_pUI;
 
             // Private tab :
