@@ -188,7 +188,7 @@ namespace nsGUI
                                                         throw (std::bad_alloc);
         virtual ~CConfigDialog () throw();
 
-        bool Run () throw();
+        bool Run (::GtkWindow*) throw();
         void Hide () throw();
 
         std::string GetConfigName () const throw();
