@@ -56,7 +56,7 @@ nsGUI::CLinkStatusTab::CLinkStatusTab () throw()
     ::GtkTable* const pTable (GTK_TABLE (::gtk_table_new (8, 2, false)));
     ::gtk_container_set_border_width (GTK_CONTAINER (pTable), 6);
     ::gtk_table_set_col_spacings (pTable, 12);
-    ::gtk_table_set_row_spacings (pTable, 8);
+    ::gtk_table_set_row_spacings (pTable, 12);
     FillRow ("Status", GTK_WIDGET (m_pStatusLabel), pTable, 0);
     FillRow ("Mode", GTK_WIDGET (m_pModeLabel), pTable, 1);
     FillRow ("Channel", GTK_WIDGET (m_pChannelLabel), pTable, 2);
@@ -72,12 +72,17 @@ nsGUI::CLinkStatusTab::CLinkStatusTab () throw()
 
 
 void nsGUI::CLinkStatusTab::SetBar (::GtkProgressBar* pBar, double BarPos,
-                                    int Value, const char* Suffix) throw()
+                        int Value, const char* Suffix, int UnknownValue) throw()
 {
     ::gtk_progress_bar_set_fraction (pBar, BarPos);
-    std::ostringstream Os;
-    Os << Value << Suffix;
-    ::gtk_progress_bar_set_text (pBar, Os.str().c_str());
+    if (Value != UnknownValue)
+    {
+        std::ostringstream Os;
+        Os << Value << Suffix;
+        ::gtk_progress_bar_set_text (pBar, Os.str().c_str());
+    }
+    else
+        ::gtk_progress_bar_set_text (pBar, "N/A");
 
 } // SetBar()
 
@@ -88,7 +93,7 @@ void nsGUI::CLinkStatusTab::Clear () throw()
     SetSignalLevel (-256);
     SetNoiseLevel (-256);
     ::gtk_label_set_text (m_pStatusLabel, DisconnectLabel);
-    ::gtk_label_set_text (m_pTxRateLabel, "");
+    ::gtk_label_set_text (m_pTxRateLabel, "N/A");
 
 } // Clear()
 
@@ -100,7 +105,7 @@ void nsGUI::CLinkStatusTab::SetChannel (unsigned Channel, double Frequency)
     if (Channel)
         Os << Channel;
     else
-        Os << "Unknown";
+        Os << "N/A";
     if (Frequency)
         Os << " (" << Frequency << " GHz)";
     ::gtk_label_set_text (m_pChannelLabel, Os.str().c_str());
