@@ -18,6 +18,7 @@
 /** \file CSelectableRowList.cxx
     \author Romain BONDUE
     \date 28/07/2005 */
+#include <cstdlib> // atoi()
 #include <cstring> // memset()
 
 #include "CSelectableRowList.h"
@@ -184,7 +185,7 @@ void nsGUI::CSelectableRowList::ToggledCallBack
     {
         ::gtk_tree_selection_select_iter (::gtk_tree_view_get_selection
                                           (This->m_pListView),
-                                           &This->m_IterVec [Path [0] - '0']);
+                                           &This->m_IterVec [atoi(Path)]);
         This->RowTriggered();
     }
 
@@ -216,7 +217,7 @@ bool nsGUI::CSelectableRowList::_RowSelectedCallBack (::GtkTreeSelection*,
 
 void nsGUI::CSelectableRowList::RowTriggered () throw()
 {
-    if (m_pUI && m_pGUICallBack && (m_pUI->*m_pGUICallBack) ())
+    if (m_pUI && m_pGUICallBack && (m_pUI->*m_pGUICallBack) (m_SelectedRow))
         for (unsigned i (0) ; i < m_IterVec.size() ; ++i)
             SetData (i, 0, int (i) == m_SelectedRow);
 
