@@ -48,7 +48,8 @@ namespace
                                                             {nsWireless::WEP};
 
     void ApplyProfile (const nsUserData::CProfile& Profile,
-                       nsWireless::CWE17Driver& Driver)
+                       nsWireless::CWE17Driver& Driver,
+                       const nsWireless::CMacAddress* pAddress = 0)
                                                     throw (nsErrors::CSystemExc)
     {
         Driver.SetSSID (Profile.GetSSID());  
@@ -64,6 +65,9 @@ namespace
                 throw;
         }
         Driver.SetEncryption (Profile.GetEncryptionD());
+        if (pAddress)
+            try{Driver.SetAPMacAddr (*pAddress);}
+            catch (const nsErrors::CSystemExc&) {} // May fail.
         try{Driver.Commit();}
             // May fail because it's not always a necessity.
         catch (const nsErrors::CSystemExc&) {}
@@ -91,7 +95,7 @@ void nsCore::CWE17Module::RecordProfiles () throw (nsErrors::CException)
 void nsCore::CWE17CLIModule::ApplyProfile (unsigned Pos)
                                                     throw (nsErrors::CSystemExc)
 {
-    ::ApplyProfile (m_ProfilesFile [Pos], m_Driver);
+    ::ApplyProfile (m_ProfilesFile.GetProfile (Pos), m_Driver);
 
 } // ApplyProfile()
 
@@ -106,22 +110,24 @@ nsCore::CWE17Module::CWE17Module (const std::string& DeviceName,
                                                 TabNoneEncrypt,
                                                 TabNoneEncryptSize,
                                                 TabSharedEncrypt,
-                                                TabSharedEncryptSize, 0, 0)),
-      m_ProfilesFile (Path)
+                                                TabSharedEncryptSize, 0, 0))
 {
     m_ProfilesFile.Extract();
 
 } // CWE17Module()
 
 
-nsUserData::CProfile* nsCore::CWE17Module::EditNewProfile () throw()
+nsUserData::CProfile* nsCore::CWE17Module::EditNewProfile
+                                             (::GtkWindow* pMainWindow) throw()
 {
-    if (!m_Editor.Edit (m_ProfilesFile.back()))
+    nsUserData::CProfile* const pProfile (&m_ProfilesFile.GetProfile
+                                                (m_ProfilesFile.Size() - 1U));
+    if (!m_Editor.Edit (pMainWindow, *pProfile))
     {
         m_ProfilesFile.DeleteLastProfile();
         return 0;
     }
-    return &m_ProfilesFile.back();
+    return pProfile;
 
 } // EditNewProfile()
 
@@ -129,38 +135,17 @@ nsUserData::CProfile* nsCore::CWE17Module::EditNewProfile () throw()
 void nsCore::CWE17Module::ApplyProfile (unsigned Pos)
                                                     throw (nsErrors::CSystemExc)
 {
-    ::ApplyProfile (m_ProfilesFile [Pos], *m_pDriver);
+    ::ApplyProfile (m_ProfilesFile.GetProfile (Pos), *m_pDriver);
 
 } // ApplyProfile()
 
 
-void nsCore::CWE17Module::Connect (const nsWireless::CCell& Cell)
-                                   throw (nsErrors::CSystemExc, std::bad_alloc)
+void nsCore::CWE17Module::ApplyProfile (unsigned Pos,
+            const nsWireless::CMacAddress& Address) throw (nsErrors::CSystemExc)
 {
-        // Only the txrate is not set, but it should be enough without it.
-    m_pDriver->SetSSID (Cell.GetSSID());
-    const nsWireless::Mode_e Mode (nsWireless::GetMatchingMode
-                                                            (Cell.GetMode()));
-    nsWireless::Mode_e CurrentMode (nsWireless::Unknown);
-    try{CurrentMode = m_pDriver->GetMode();}
-    catch (...) {} // Ignore everything.
-    if (CurrentMode != Mode)
-        m_pDriver->SetMode (Mode);
-    try{m_pDriver->SetChannel (Cell.GetChannel());}
-    catch (const nsErrors::CSystemExc&)
-    {       // May be fine in managed mode.
-        if (Mode != nsWireless::Managed)
-            throw;
-    }
-    m_pDriver->SetEncryption (Cell.GetEncryptionD());
-        // May fail because it's not always a necessity.
-    try {m_pDriver->SetAPMacAddr (Cell.GetAPMacAddress());}
-    catch (const nsErrors::CSystemExc&) {}
-        // May fail because it's not always a necessity.
-    try {m_pDriver->Commit();}
-    catch (const nsErrors::CSystemExc&) {}
+    ::ApplyProfile (m_ProfilesFile.GetProfile (Pos), *m_pDriver, &Address);
 
-} // Connect()
+} // ApplyProfile()
 
 
 void nsCore::CWE17Module::BecomeRoot () throw (nsErrors::CException,
