@@ -35,7 +35,7 @@ namespace
         std::istringstream Is (Str);
         T Value;
         Is >> Value;
-        if (Is || Is.eof()) return Value;
+        if (Is) return Value;
         throw nsErrors::CException ("Invalid data read (from helper).",
                                     nsErrors::InvalidDataFromHelper);
 
@@ -50,6 +50,9 @@ namespace
         unsigned DefaultKey;
         std::istringstream Is (Str);
         Is >> Encrypt >> Auth >> DefaultKey;
+        if (!Is)
+            throw nsErrors::CException ("Invalid data read (from helper).",
+                                        nsErrors::InvalidDataFromHelper);
         nsWireless::CEncryptionD Descriptor;
         Descriptor.SetEncrypt (nsWireless::EncryptType_e (Encrypt));
         Descriptor.SetAuth (nsWireless::AuthType_e (Auth));
@@ -70,7 +73,7 @@ namespace
                                             nsErrors::InvalidDataFromHelper);
             }
         }
-        if (!(Is || Is.eof()))
+        if (!Is)
             throw nsErrors::CException ("Invalid data read (from helper).",
                                         nsErrors::InvalidDataFromHelper);
         return Descriptor;
