@@ -296,6 +296,8 @@ nsGUI::CGtkGUI::CGtkGUI () throw (nsErrors::CException)
                                         this);
     m_PrefsPage.AddTrayIconButtonCallBack
                             (G_CALLBACK (DisplayTrayIconButtonCallBack), this);
+    m_PrefsPage.AddStartMinimizedCallBack
+                            (G_CALLBACK (StartMinimizedButtonCallBack), this);
     m_PrefsPage.AddIfaceStatusChangeCallBack
                                 (G_CALLBACK (IfaceStatusChangedCallBack), this);
 
@@ -311,23 +313,13 @@ nsGUI::CGtkGUI::CGtkGUI () throw (nsErrors::CException)
 } // CGtkGUI()
 
 
-nsGUI::CGtkGUI::~CGtkGUI () throw (nsErrors::CException)
+nsGUI::CGtkGUI::~CGtkGUI () throw()
 {
-    try
-    {
-        if (m_pModule.get())
-            StoreData();
-    }
-    catch (const nsErrors::CException&)
-    {
-        ::gtk_widget_destroy (GTK_WIDGET (m_pWindow));
-        if (m_GlobalTimer) ::g_source_remove (m_GlobalTimer);
-        if (m_SiteSurveyTabTimer) ::g_source_remove (m_SiteSurveyTabTimer);
-        throw;
-    }
     ::gtk_widget_destroy (GTK_WIDGET (m_pWindow));
-    if (m_GlobalTimer) ::g_source_remove (m_GlobalTimer);
-    if (m_SiteSurveyTabTimer) ::g_source_remove (m_SiteSurveyTabTimer);
+    if (m_GlobalTimer)
+        ::g_source_remove (m_GlobalTimer);
+    if (m_SiteSurveyTabTimer)
+        ::g_source_remove (m_SiteSurveyTabTimer);
     if (m_pTrayMenu)
         ::gtk_object_destroy (GTK_OBJECT (m_pTrayMenu));
 
@@ -877,6 +869,7 @@ bool nsGUI::CGtkGUI::DeleteProfile (unsigned ProfileI) throw()
     if (Response == GTK_RESPONSE_YES)
     {
         m_pModule->DeleteProfile (ProfileI);
+        StoreProfiles();
         return true; // Delete the profile from the list.
     }
     return false; // Don't delete the profile from the list.
@@ -1347,19 +1340,23 @@ void nsGUI::CGtkGUI::ChangeInterfaceCallBack (CGtkGUI* This) throw()
 
 void nsGUI::CGtkGUI::AutoLeaveRootModeCallBack (CGtkGUI* This) throw()
 {
-    if (!This->IsIgnored() && This->IsRoot())
+    if (!This->IsIgnored())
     {
-        if (This->m_LeaveRootModeTimer)
+        if (This->IsRoot())
         {
-            ::g_source_remove (This->m_LeaveRootModeTimer);
-            This->m_LeaveRootModeTimer = 0;
-        }
-        const unsigned Delay (This->m_PrefsPage.GetAutoLeaveRootDelay() *
+            if (This->m_LeaveRootModeTimer)
+            {
+                ::g_source_remove (This->m_LeaveRootModeTimer);
+                This->m_LeaveRootModeTimer = 0;
+            }
+            const unsigned Delay (This->m_PrefsPage.GetAutoLeaveRootDelay() *
                                                                         1000);
-        if (Delay)
-            This->m_LeaveRootModeTimer = ::g_timeout_add (Delay,
+            if (Delay)
+                This->m_LeaveRootModeTimer = ::g_timeout_add (Delay,
                                      reinterpret_cast< ::gboolean (*) (void*)>
-                                        (LeaveRootModeTimerCallBack), This);
+                                            (LeaveRootModeTimerCallBack), This);
+        }
+        This->StoreOptions();
     }
 
 } // AutoLeaveRootModeCallBack()
@@ -1507,11 +1504,20 @@ void nsGUI::CGtkGUI::DisplayTrayIconButtonCallBack (CGtkGUI* This) throw()
                                     (GlobalTimerWithoutTrayCallBack), This);
             GlobalTimerWithoutTrayCallBack (This);
         }
+        This->StoreOptions();
     }
 
 } // DisplayTrayIconButtonCallBack()
 
 
+void nsGUI::CGtkGUI::StartMinimizedButtonCallBack (CGtkGUI* This) throw()
+{
+    if (!This->IsIgnored())
+        This->StoreOptions();
+
+} // StartMinimizedButtonCallBack()
+
+
 void nsGUI::CGtkGUI::LoadOptions () throw()
 {
         // The constructor parameters are the default settings.
@@ -1527,7 +1533,22 @@ void nsGUI::CGtkGUI::LoadOptions () throw()
 } // LoadOptions()
 
 
-void nsGUI::CGtkGUI::StoreData () throw (nsErrors::CException)
+void nsGUI::CGtkGUI::StoreProfiles () throw()
+{
+    try
+    {
+        m_pModule->RecordProfiles();
+    }
+    catch (const nsErrors::CException& Exc)
+    {
+        NotifyError ("Profiles could not have been recorded.", Exc.GetCode(),
+                                                                    m_pWindow);
+    }
+
+} // StoreProfiles()
+
+
+void nsGUI::CGtkGUI::StoreOptions () throw()
 {
     try
     {
@@ -1535,16 +1556,14 @@ void nsGUI::CGtkGUI::StoreData () throw (nsErrors::CException)
                                               m_PrefsPage.StartMinimized(),
                                           m_PrefsPage.GetAutoLeaveRootDelay());
         OptionsFile.Record();
-        m_pModule->RecordProfiles();
     }
     catch (const nsErrors::CException& Exc)
     {
-        NotifyError ("Options and profiles could not have been recorded.",
-                     Exc.GetCode(), m_pWindow);
-        throw;
+        NotifyError ("Options could not have been recorded.", Exc.GetCode(),
+                     m_pWindow);
     }
 
-} // StoreData()
+} // StoreOptions()
 
 
 void nsGUI::CGtkGUI::IfaceStatusChangedCallBack (CGtkGUI* This) throw()
@@ -1736,8 +1755,33 @@ void nsGUI::CGtkGUI::SelectMatchingProfile (unsigned ProfileI) throw()
 } // ProfileI()
 
 
-void nsGUI::CGtkGUI::ApplyProfileTrayCallBack (::gpointer ProfileI) throw()
+void nsGUI::CGtkGUI::ApplyProfileTrayCallBack (::gpointer ProfileIPtr) throw()
 {
-    GlobalUglyThis->ApplyProfile (GPOINTER_TO_INT (ProfileI));
+    const ::gint ProfileI (GPOINTER_TO_INT (ProfileIPtr));
+    if (GlobalUglyThis->ApplyProfile (ProfileI))
+        GlobalUglyThis->SelectMatchingProfile (ProfileI);
 
 } // ApplyProfileTrayCallBack()
+
+
+const nsUserData::CProfile* nsGUI::CGtkGUI::EditProfile () throw()
+{
+    if (m_pModule->EditProfile (m_pWindow, m_ProfileTab.GetSelectedRow()))
+    {
+        StoreProfiles();
+        return &m_pModule->GetProfile (m_ProfileTab.GetSelectedRow());
+    }
+    return 0;
+
+} // EditProfile()
+
+
+const nsUserData::CProfile* nsGUI::CGtkGUI::NewProfile () throw()
+{
+    const nsUserData::CProfile* const pProfile (m_pModule->NewProfile
+                                                                (m_pWindow));
+    if (pProfile)
+        StoreProfiles();
+    return pProfile;
+
+} // NewProfile()
