@@ -129,9 +129,10 @@ void nsNet::CInterface::SetIPFromDHCP (const std::string& Data)
 {
     if (nsSystem::Fork())
     {
-        const char* Argv [5] = {(nsCore::DhcpScriptPrefix +
-                                               nsCore::DhcpScriptName).c_str(),
-                                m_IfName.c_str(), Data.c_str(), 0};
+        const std::string Path (nsCore::DhcpScriptPrefix +
+                                nsCore::DhcpScriptName);
+        const char* Argv [5] = {Path.c_str(), m_IfName.c_str(), Data.c_str(),
+                                0};
         try {nsSystem::Exec (Argv);}
         catch (const nsErrors::CException& Exc)
         {
@@ -179,8 +180,11 @@ void nsNet::CInterface::Up () throw (nsErrors::CSystemExc)
 {
     ::ifreq Req;
     GetInterfaceFlags (Req, m_IfName);
-    Req.ifr_flags |= IFF_UP;
-    nsSystem::Ioctl (SIOCSIFFLAGS, &Req, "Can't bring interface up.");
+    if (!(Req.ifr_flags & IFF_UP))
+    {
+        Req.ifr_flags |= IFF_UP | IFF_RUNNING;
+        nsSystem::Ioctl (SIOCSIFFLAGS, &Req, "Can't bring interface up.");
+    }
 
 } // Up()
 
@@ -189,7 +193,10 @@ void nsNet::CInterface::Down () throw (nsErrors::CSystemExc)
 {
     ::ifreq Req;
     GetInterfaceFlags (Req, m_IfName);
-    nsSystem::Ioctl (SIOCSIFFLAGS, &Req, "Can't bring interface down.");
-    Req.ifr_flags &= ~IFF_UP;
+    if (Req.ifr_flags & IFF_UP)
+    {
+        Req.ifr_flags &= ~(IFF_UP | IFF_RUNNING);
+        nsSystem::Ioctl (SIOCSIFFLAGS, &Req, "Can't bring interface down.");
+    }
 
 } // Down()
