@@ -60,7 +60,8 @@ namespace
     const std::string WEPName ("WEP");
     const std::string AESName ("AES");
     const std::string TKIPName ("TKIP");
-    const std::string UnknownEncryptNTCTS ("UNKNOW");
+    const std::string TKIPAESName ("TKIP, AES");
+    const std::string UnknownEncryptNTCTS ("UNKNOWN");
 
     const std::string OpenName ("OPEN");
     const std::string SharedName ("SHARED");
@@ -69,7 +70,7 @@ namespace
     const std::string WPA2PSKName ("WPA2PSK");
     const std::string WPAName ("WPA");
     const std::string WPA2Name ("WPA2");
-    const std::string UnknownAuthNTCTS ("UNKNOW");
+    const std::string UnknownAuthNTCTS ("UNKNOWN");
 
 } // anonymous namespace
 
@@ -126,6 +127,7 @@ const std::string& nsWireless::GetEncryptName (EncryptType_e Encrypt) throw()
       case WEP : return WEPName;
       case AES : return AESName;
       case TKIP : return TKIPName;
+      case TKIPAES : return TKIPAESName;
       default : return UnknownEncryptNTCTS;
     }
 
@@ -139,6 +141,7 @@ nsWireless::EncryptType_e nsWireless::GetEncryptFromName
     if (Name == NoneName) return None;
     if (Name == TKIPName) return TKIP;
     if (Name == AESName) return AES;
+    if (Name == TKIPAESName) return TKIPAES;
     return UnknownEnc;
 
 } // GetEncryptFromName()
@@ -223,7 +226,7 @@ void nsWireless::CHexaKey::Set (std::string Key) throw (CBadFormatExc)
     for (std::string::size_type i (Key.size()) ; i-- ; )
         if (Key [i] == '-' || Key [i] == ':' || Key [i] == ' ')
             Key.erase (i, 1);
-        else if (!isxdigit (Key [i]))
+        else if (!isxdigit (static_cast<unsigned char> (Key[i])))
         {
 #ifndef NDEBUG
             std::cerr << "Bad format, key : #" << Key << "# byte " << i
@@ -232,7 +235,7 @@ void nsWireless::CHexaKey::Set (std::string Key) throw (CBadFormatExc)
             throw CBadFormatExc();
         }
         else
-            Key [i] = toupper (Key [i]);
+            Key [i] = toupper (static_cast<unsigned char> (Key[i]));
     if (Key.empty())
     {
 #ifndef NDEBUG
@@ -352,11 +355,19 @@ nsWireless::CEncryptionD& nsWireless::CEncryptionD::operator =
 bool nsWireless::CEncryptionD::operator == (const CEncryptionD& D) const
                                                                         throw()
 {
-    bool IsEqual (m_Auth == D.m_Auth && m_Encrypt == D.m_Encrypt);
-    for (unsigned I (0) ; IsEqual && I < MaxNbKey ; ++I)
-        IsEqual = m_KeyTab [I] == D.m_KeyTab [I];
-
-    return IsEqual;
+    if (m_Auth == D.m_Auth && m_Encrypt == D.m_Encrypt ||
+                                            m_Encrypt == nsWireless::TKIPAES && 
+                                            (D.m_Encrypt == nsWireless::TKIP ||
+                                             D.m_Encrypt == nsWireless::AES) ||
+                                        D.m_Encrypt == nsWireless::TKIPAES &&
+                (m_Encrypt == nsWireless::TKIP || m_Encrypt == nsWireless::AES))
+    {
+        for (unsigned I (0) ; I < MaxNbKey ; ++I)
+            if (m_KeyTab [I] != D.m_KeyTab [I])
+                return false;
+        return true;
+    }
+    return false;
 
 } // operator == ()
 
