@@ -28,7 +28,8 @@
 nsCore::CProfileEditor::~CProfileEditor () throw() {}
 
 
-bool nsCore::CProfileEditor::Edit (nsUserData::CProfile& Profile) throw()
+bool nsCore::CProfileEditor::Edit (::GtkWindow* pMainWindow,
+				   nsUserData::CProfile& Profile) throw()
 {
     m_pDialog->SetConfigName (Profile.GetName());
     m_pDialog->SetSSID (Profile.GetSSID());
@@ -42,7 +43,7 @@ bool nsCore::CProfileEditor::Edit (nsUserData::CProfile& Profile) throw()
     m_pDialog->SetIPSetting (Profile.GetIPSettings().GetSetting());
 
     bool IsModified (false);
-    while (m_pDialog->Run())
+    while (m_pDialog->Run (pMainWindow))
     {
         const std::string Name (m_pDialog->GetConfigName());
         const std::string SSID (m_pDialog->GetSSID());
