@@ -21,11 +21,13 @@
 #include <cstring>
 #include <cstdio>
 #include <sstream>
+#include <cstring>
 #ifndef NDEBUG
 #include <iostream>
 #endif // NDEBUG
 
 extern "C"{
+#include <stdint.h>
 #include <sys/ioctl.h> // SIOCDEVPRIVATE
 }
 
@@ -36,6 +38,39 @@ extern "C"{
 
 namespace
 {
+        // FIXME The two wep ciphers are not differenciated.
+        //       WPA 1 parsing code can be simplified like WPA 2.
+        //       Both mode actually use the same constants (but a different
+        //       prefix section).
+        //       Parsing is quite alike in both case (same bit layout, mostly).
+        // From wpa_supplicant wpa.c :
+    const ::uint8_t WPA_OUI_TYPE[] = {0x00, 0x50, 0xf2, 1};
+    const ::uint8_t WPA_AUTH_KEY_MGMT_NONE[] = {0x00, 0x50, 0xf2, 0};
+    const ::uint8_t WPA_AUTH_KEY_MGMT_UNSPEC_802_1X[] = {0x00, 0x50, 0xf2, 1};
+    const ::uint8_t WPA_AUTH_KEY_MGMT_PSK_OVER_802_1X[] = {0x00, 0x50, 0xf2, 2};
+    const ::uint8_t WPA_CIPHER_SUITE_NONE[] = {0x00, 0x50, 0xf2, 0};
+    const ::uint8_t WPA_CIPHER_SUITE_WEP40[] = {0x00, 0x50, 0xf2, 1};
+    const ::uint8_t WPA_CIPHER_SUITE_TKIP[] = {0x00, 0x50, 0xf2, 2};
+    const ::uint8_t WPA_CIPHER_SUITE_WRAP[] = {0x00, 0x50, 0xf2, 3};
+    const ::uint8_t WPA_CIPHER_SUITE_CCMP[] = {0x00, 0x50, 0xf2, 4};
+    const ::uint8_t WPA_CIPHER_SUITE_WEP104[] = {0x00, 0x50, 0xf2, 5};
+
+        // Adapted from wpa_supplicant wpa.c :
+    const ::uint8_t RSN_SEQUENCE_SIZE (4U);
+    const ::uint8_t RSN_PREFIX_SIZE (RSN_SEQUENCE_SIZE - 1U);
+    const ::uint8_t RSN_PREFIX [RSN_SEQUENCE_SIZE - 1U] = {0U, 0xfU, 0xacU};
+    const ::uint8_t RSN_AUTH_UNSPEC_802_1X (1U);
+    const ::uint8_t RSN_AUTH_PSK (2U);
+    const ::uint8_t RSN_CIPHER_NONE (0U);
+    const ::uint8_t RSN_CIPHER_WEP40 (1U);
+    const ::uint8_t RSN_CIPHER_TKIP (2U);
+    const ::uint8_t RSN_CIPHER_CCMP_WRAP (3U);
+    const ::uint8_t RSN_CIPHER_CCMP (4U);
+    const ::uint8_t RSN_CIPHER_WEP104 (5U);
+    const ::uint8_t RSN_CIPHER_AES_CMAC (6U); // FIXME Unsupported currently.
+
+
+
     class CScanResult
     {
       protected :
@@ -65,24 +100,261 @@ namespace
 
         }; // SConfig
 
-        struct SEncryptionD
+        struct SIE
         {
+            typedef ::uint8_t DataType_t;
+            static const unsigned DataSize = 32U; 
+
             unsigned char Id;
             unsigned char Length;
-            unsigned char Data [32];
+            DataType_t Data [DataSize];
 
-        }; // SEncryptionD
+        }; // SIE
 
-        struct SEncryptionOffset
+        struct SFixedIE
         {
             unsigned char m1 [8];
             unsigned short m2;
             unsigned short m3;
 
-        }; // SEncryptionOffset
+        }; // SFixedIE
 
 
       private :
+        bool NextData (const SIE::DataType_t*& pData, int& LengthRemaining)
+                                                                const throw()
+        {
+            if (LengthRemaining > RSN_SEQUENCE_SIZE && !std::memcmp (pData,
+                                                RSN_PREFIX, RSN_PREFIX_SIZE))
+            {
+                pData += RSN_PREFIX_SIZE;
+                LengthRemaining -= RSN_PREFIX_SIZE;
+                return true;
+            }
+            return false;
+
+        } // NextData()
+
+            // ? FIXME ? A CIE class would be clean (it's not Ralink specific).
+            // It would be more lisible with sub function : parse_cipher() ...
+        void ParseIE (const SIE& IE, nsWireless::CEncryptionD& Descriptor) const
+                                                                        throw()
+        {
+            int LengthRemaining (IE.Length);
+            const SIE::DataType_t* pDataIter (IE.Data);
+            if (IE.Id == 0xDDU && LengthRemaining >=
+                                                    int (sizeof (WPA_OUI_TYPE)))
+            {
+                const size_t VersionSize (2U);
+                if (!std::memcmp(pDataIter, WPA_OUI_TYPE,
+                                                        sizeof (WPA_OUI_TYPE)))
+                {
+                    int Offset (sizeof (WPA_OUI_TYPE) + VersionSize +
+                                                sizeof (WPA_CIPHER_SUITE_TKIP));
+                    LengthRemaining -= Offset;
+                    Descriptor.SetAuth (nsWireless::WPAPSK);
+                    Descriptor.SetEncrypt (nsWireless::TKIP);
+                    const int CipherCountSize (2U);
+                    if (LengthRemaining >= CipherCountSize)
+                    {
+                        LengthRemaining -= CipherCountSize;
+                        pDataIter += Offset;
+                        const SIE::DataType_t CipherCount (*pDataIter);
+                        Offset = CipherCount * sizeof (WPA_CIPHER_SUITE_TKIP);
+                        if (LengthRemaining >= Offset)
+                        {
+                            pDataIter += CipherCountSize;
+                            switch (CipherCount)
+                            {
+                              case 1 :
+                                if (!std::memcmp (pDataIter,
+                                                        WPA_CIPHER_SUITE_CCMP,
+                                                sizeof (WPA_CIPHER_SUITE_CCMP)))
+                                    Descriptor.SetEncrypt (nsWireless::AES);
+                                else if (!std::memcmp (pDataIter,
+                                                        WPA_CIPHER_SUITE_WEP40,
+                                            sizeof (WPA_CIPHER_SUITE_WEP40)) ||
+                                                        !std::memcmp (pDataIter,
+                                                        WPA_CIPHER_SUITE_WEP104,
+                                            sizeof (WPA_CIPHER_SUITE_WEP104))
+                                        )
+                                    Descriptor.SetEncrypt (nsWireless::WEP);
+                                else if (std::memcmp (pDataIter,
+                                                        WPA_CIPHER_SUITE_TKIP,
+                                                sizeof (WPA_CIPHER_SUITE_TKIP)))
+                                    Descriptor.SetEncrypt
+                                                    (nsWireless::UnknownEnc);
+                              break;
+
+                              case 2 :
+                              {      
+                                const SIE::DataType_t* const pSecondCipher
+                                                                    (pDataIter +
+                                             sizeof (WPA_CIPHER_SUITE_TKIP));
+                                if (!memcmp (pDataIter, WPA_CIPHER_SUITE_TKIP,
+                                            sizeof (WPA_CIPHER_SUITE_TKIP)) &&
+                                                        !memcmp (pSecondCipher,
+                                                        WPA_CIPHER_SUITE_CCMP,
+                                            sizeof (WPA_CIPHER_SUITE_CCMP)) ||
+                                    !memcmp (pDataIter, WPA_CIPHER_SUITE_CCMP,
+                                            sizeof (WPA_CIPHER_SUITE_CCMP)) &&
+                                                        !memcmp (pSecondCipher,
+                                                        WPA_CIPHER_SUITE_TKIP,
+                                                sizeof (WPA_CIPHER_SUITE_TKIP)))
+                                    Descriptor.SetEncrypt (nsWireless::TKIPAES);
+                                else // Not completely true, we may know one.
+                                    Descriptor.SetEncrypt
+                                                    (nsWireless::UnknownEnc);
+                              }
+                              break;
+
+                              default :
+                                Descriptor.SetEncrypt (nsWireless::UnknownEnc);
+                            }
+                            LengthRemaining -= Offset;
+                            const int AuthCountSize (2U);
+                            bool AuthSet (false);
+                            if (LengthRemaining >= AuthCountSize)
+                            {
+                                pDataIter += Offset;
+                                const SIE::DataType_t AuthCount (*pDataIter);
+                                pDataIter += AuthCountSize;
+                                Offset = AuthCount *
+                                    sizeof (WPA_AUTH_KEY_MGMT_UNSPEC_802_1X);
+                                if (LengthRemaining >= Offset)
+                                    for (size_t Cnt (0) ; Cnt < AuthCount ;
+                                                                        ++Cnt)
+                                    {
+                                        if (!std::memcmp (pDataIter,
+                                                WPA_AUTH_KEY_MGMT_UNSPEC_802_1X,
+                                                                        sizeof
+                                        (WPA_AUTH_KEY_MGMT_UNSPEC_802_1X)) &&
+                                                /* NOTE : Legacy drivers stuff
+                                                 * here, we try to use WPAPSK
+                                                 * if possible because these
+                                                 * drivers only support this. */
+                                                                    !AuthSet)
+                                        {
+                                            Descriptor.SetAuth
+                                                            (nsWireless::WPA);
+                                            AuthSet = true;
+                                        }
+                                        else if (!std::memcmp (pDataIter,
+                                            WPA_AUTH_KEY_MGMT_PSK_OVER_802_1X,
+                                                                        sizeof
+                                        (WPA_AUTH_KEY_MGMT_PSK_OVER_802_1X)))
+                                        {
+                                            Descriptor.SetAuth
+                                                        (nsWireless::WPAPSK);
+                                            AuthSet = true;
+                                        }
+                                    }
+                            }
+                            if (!AuthSet)
+                                Descriptor.SetAuth (nsWireless::UnknownAuth);
+                        }
+                    }
+                }
+            }
+            else if (IE.Id == 0x30 && pDataIter [0U] == 1U ||
+                                                        pDataIter [1U] == 1U)
+            {
+                const size_t VersionSize (2U);
+                pDataIter += VersionSize;
+                LengthRemaining -= VersionSize;
+                Descriptor.SetAuth (nsWireless::WPA2);
+                Descriptor.SetEncrypt (nsWireless::AES);
+                const size_t CipherCountSize (2U);
+                LengthRemaining -= CipherCountSize; // Trick, NextData() will
+                if (NextData (pDataIter, LengthRemaining)) // check for it!
+                {
+                    const size_t CipherCount (*++pDataIter);
+                    pDataIter += CipherCountSize;
+                    if (NextData (pDataIter, LengthRemaining))
+                    {
+                        /*
+                            // FIXME Won't actually report several network.
+                    while (CipherCount && NextData (pDataIter, LengthRemaining))
+                    {
+                        switch (*pDataIter)
+                        {      Sort the cases by likeliness (arbitrary).
+                          case RSN_CIPHER_WEP40 :
+                            Do something ...
+                          break;
+                           etc ...
+                        }
+                        --CipherCount;
+                        ++pDataIter; // TODO Remove the incrementation in
+                                             NextData() next call.
+                    }
+                        */
+                        nsWireless::EncryptType_e Cipher
+                                                    (nsWireless::UnknownEnc);
+                        if (CipherCount == 1U)
+                            switch (*pDataIter)
+                            {
+                              case RSN_CIPHER_CCMP_WRAP :
+                              case RSN_CIPHER_CCMP :
+                                Cipher = nsWireless::AES;
+                              break;
+
+                              case RSN_CIPHER_TKIP :
+                                Cipher = nsWireless::TKIP;
+                              break;
+
+                              case RSN_CIPHER_WEP40 :
+                              case RSN_CIPHER_WEP104 :
+                                Cipher = nsWireless::WEP;
+                              break;
+
+                              case RSN_CIPHER_NONE :
+                                Cipher = nsWireless::None;
+                              //break;
+                            }
+                            // FIXME Assumption, no check :
+                        else if (CipherCount == 2U && NextData (++pDataIter,
+                                                            LengthRemaining))
+                            Cipher = nsWireless::TKIPAES;
+                        Descriptor.SetEncrypt (Cipher);
+                        const size_t AuthCountSize (2U);
+                        LengthRemaining -= AuthCountSize; // Same trick.
+                        if (NextData (++pDataIter, LengthRemaining))
+                        {
+                            size_t AuthCount (*++pDataIter);
+                            pDataIter += AuthCountSize;
+                            nsWireless::AuthType_e Auth
+                                                    (nsWireless::UnknownAuth);
+                            while (AuthCount && NextData (pDataIter,
+                                                            LengthRemaining))
+                            {
+                                switch (*pDataIter)
+                                {
+                                  case RSN_AUTH_PSK :
+                                    Auth = nsWireless::WPA2PSK;
+                                  break;
+
+                                  case RSN_AUTH_UNSPEC_802_1X :
+                                /* NOTE : Legacy drivers stuff here, we favor
+                                 * WPA2PSK because these drivers only support
+                                 * this. */
+                                    if (Auth != nsWireless::WPA2PSK)
+                                        Auth = nsWireless::WPA2;
+                                  //break;
+                                }
+                                --AuthCount;
+                                ++pDataIter;
+                            }
+                            Descriptor.SetAuth (Auth);
+                            // There may be other stuff in the IE. We don't
+                            // currently need or use them.
+                        }
+                    }
+                }
+            }
+
+        } // ParseIE()
+
+
         static const unsigned NbRates = 16;
 
         unsigned m_Length;
@@ -96,8 +368,7 @@ namespace
         int m_NetworkInfrastructureType;
         unsigned char m_SupportedRates [NbRates];
         unsigned m_IELength;
-        unsigned char m_pIEs [sizeof (SEncryptionOffset) +
-                                                  sizeof (SEncryptionD) * 4];
+        unsigned char m_pIEs [sizeof (SFixedIE) + sizeof (SIE) * 4];
 
 
       public :
@@ -127,49 +398,42 @@ namespace
         {
             nsWireless::CEncryptionD Descriptor;
             if (m_Privacy)
-            {       // Check if it's WPA :
-                const SEncryptionD* const pDescriptor
-                                (reinterpret_cast<const SEncryptionD* const>
-                                        (m_pIEs + sizeof (SEncryptionOffset)));
+            {
 #ifndef NDEBUG
-                std::cerr << std::hex << "Ralink magic encryption function :\n"
-                             "\tpDescriptor->Id : "
-                          << unsigned (pDescriptor->Id)
-                          << "\n\tpDescriptor->Length : "
-                          << unsigned (pDescriptor->Length)
-                          << "\n\tpDescriptor->Data : "
-                          << *reinterpret_cast<const unsigned*>
-                                                          (pDescriptor->Data)
-                          << "\n\tpDescriptor->Data [9] : "
-                          << unsigned (pDescriptor->Data [9]) << std::dec
+                std::cerr << "Ralink encryption, privacy enabled, essid : "
+                          << std::string (reinterpret_cast<const char*>
+                                              (m_SSID.Text), 0, m_SSID.Length)
                           << std::endl;
 #endif // NDEBUG
-                if (pDescriptor->Id == 221 && pDescriptor->Length >= 16 &&
-                    *reinterpret_cast<const unsigned*> (pDescriptor->Data)
-                                                            == 0x01F25000)
+                Descriptor.SetEncrypt (nsWireless::WEP);
+                if (m_IELength > sizeof (SFixedIE))
                 {
-                    switch (pDescriptor->Data [9])
+                    const SIE* pDescriptor
+                                    (reinterpret_cast<const SIE* const>
+                                                (m_pIEs + sizeof (SFixedIE)));
+                    for (size_t LengthRemaining (m_IELength -
+                                                            sizeof (SFixedIE)) ;
+                                                            LengthRemaining ; )
                     {
-                      case 2 :
-                        Descriptor.SetEncrypt (nsWireless::TKIP);
-                      break;
-
-                      case 3 : // AES-WRAP
-                      case 4 : // AES-CCMP
-                        Descriptor.SetEncrypt (nsWireless::AES);
-                      //break;
+#ifndef NDEBUG
+                        std::cerr << std::hex << "\tpDescriptor->Id : "
+                                  << unsigned (pDescriptor->Id)
+                                  << "\n\tpDescriptor->Length : "
+                                  << unsigned (pDescriptor->Length)
+                                  << std::endl;
+                        for (unsigned I (0U) ; I < pDescriptor->Length ; ++I)
+                            std::cerr << unsigned (pDescriptor->Data [I])
+                                      << ' ';
+                        std::cerr << std::dec << std::endl;
+#endif // NDEBUG
+                        ParseIE (*pDescriptor, Descriptor);
+                        LengthRemaining -= pDescriptor->Length + sizeof (SIE) -
+                                    sizeof (SIE::DataType_t) * SIE::DataSize;
+                        pDescriptor = reinterpret_cast<const SIE*>
+                                                (reinterpret_cast<const char*>
+                                     (pDescriptor->Data) + pDescriptor->Length);
                     }
-                    if (m_NetworkInfrastructureType)
-                        Descriptor.SetAuth (nsWireless::WPAPSK);
-                    else
-                        Descriptor.SetAuth (nsWireless::WPANONE);
                 }
-                else Descriptor.SetEncrypt (nsWireless::WEP);
-                    /* FIXME We don't know if it's "shared" or not.
-                     * Dig more in Ralink stuffs? (It seems that the driver
-                     * just not reports it : get_site_survey (rt61/73) does
-                     * not have that piece of information too.)
-                     * RaConfig on Windows neither, *I think*. */
             }
             return Descriptor;
 
@@ -285,13 +549,8 @@ namespace
             AuthType = nsWireless::WPA2PSK;
         else
             AuthType = nsWireless::GetAuthFromName (Auth);
-        nsWireless::EncryptType_e EncType;
-        if (!strcmp (Enc, "TKIP, AES"))
-            EncType = nsWireless::AES;
-        else
-            EncType = nsWireless::GetEncryptFromName (Enc);
         return nsWireless::CEncryptionD (nsWireless::CHexaKey(), AuthType,
-                                         EncType);
+                                         nsWireless::GetEncryptFromName (Enc));
 
     } // ToEncryptionDRt61()
 
@@ -312,6 +571,21 @@ namespace
     } // ToQuality()
 
 
+        // Assumes that BufSize is stricly superior to one, and Dest has, at
+        // least, enough room for BufSize + 1 characters.
+    void Extract (const char* Src, char* Dest, int BufSize) throw()
+    {
+        while (--BufSize >= 0 && Src [BufSize] == ' ');
+        Dest [BufSize + 1] = '\0';
+        while (BufSize >= 0)
+        {
+            Dest[BufSize] = Src[BufSize];
+            --BufSize;
+        }
+
+    } // Extract()
+
+
 
         // 8 8 36 20 12 12 12
         // Channel RSSI SSID Bssid WepStatus AuthMode NetworkType
@@ -323,11 +597,10 @@ namespace
             throw nsErrors::CException ("Data parsing error (1).",
                                         nsErrors::DataParsingError1);
         Line += 17;
-        int End (35);
-        while (End >= 0 && Line [End] == ' ')
-            --End;
-        const std::string Ssid (Line, 0, End + 1);
-        Line += 36;
+        const int SsidSize (36);
+        char Ssid [SsidSize + 1];
+        Extract(Line, Ssid, SsidSize);
+        Line += SsidSize;
         char Bssid [21];
         char Enc [13];
         char Auth [13];
@@ -336,9 +609,9 @@ namespace
             throw nsErrors::CException ("Data parsing error (2).",
                                         nsErrors::DataParsingError2);
 #ifndef NDEBUG
-        std::cerr << '\t' << Channel << ' ' << Rssi << " #" << Ssid << "# "
-                  << Bssid << ' ' << Enc << ' ' << Auth << ' ' << NetworkType
-                  << std::endl;
+        std::cerr << "Parsed data : " << Channel << ' ' << Rssi << " #" << Ssid
+                  << "# " << Bssid << ' ' << Enc << ' ' << Auth << ' '
+                  << NetworkType << std::endl;
 #endif // NDEBUG
         return nsWireless::CCell (nsWireless::CMacAddress (std::string (Bssid)),
                                   ToMode (NetworkType), Ssid,
@@ -358,25 +631,29 @@ namespace
             throw nsErrors::CException ("Data parsing error (1).",
                                         nsErrors::DataParsingError1);
         Line += 17;
-#ifndef NDEBUG
-        std::cerr << "Line : " << Line << std::endl;
-#endif // NDEBUG
-        int End (35);
-        while (End >= 0 && Line [End] == ' ')
-            --End;
-        const std::string Ssid (Line, 0, End + 1);
-        Line += 36;
-        char Bssid [21];
-        char Enc [13];
-        char Auth [21];
-        char NetworkType [13];
-        if (sscanf (Line, "%s%s%s%s", Bssid, Enc, Auth, NetworkType) != 4)
+        const int SsidSize (36);
+        char Ssid [SsidSize + 1];
+        Extract(Line, Ssid, SsidSize);
+        Line += SsidSize;
+        const int BssidSize (20);
+        char Bssid [BssidSize + 1];
+        if (sscanf (Line, "%s", Bssid) != 1)
             throw nsErrors::CException ("Data parsing error (2).",
                                         nsErrors::DataParsingError2);
+        Line += BssidSize;
+        const int EncSize (12);
+        char Enc [EncSize + 1];
+        Extract(Line, Enc, EncSize);
+        Line += EncSize;
+        char Auth [21];
+        char NetworkType [13];
+        if (sscanf (Line, "%s%s", Auth, NetworkType) != 2)
+            throw nsErrors::CException ("Data parsing error (3).",
+                                        nsErrors::DataParsingError3);
 #ifndef NDEBUG
-        std::cerr << '\t' << Channel << ' ' << Rssi << " #" << Ssid << "# "
-                  << Bssid << ' ' << Enc << ' ' << Auth << ' ' << NetworkType
-                  << std::endl;
+        std::cerr << "Parsed data : " << Channel << ' ' << Rssi << " #" << Ssid
+                  << "# " << Bssid << ' ' << Enc << ' ' << Auth << ' '
+                  << NetworkType << std::endl;
 #endif // NDEBUG
         return nsWireless::CCell (nsWireless::CMacAddress (std::string (Bssid)),
                                   ToMode (NetworkType), Ssid,
@@ -450,42 +727,15 @@ void nsWireless::CRT2400Driver::Scan () throw (nsErrors::CSystemExc)
 void nsWireless::CRTAdvancedDriver::GetScanResult (u16 MagicNumber,
 std::vector<CCell>& CellVec) const throw (nsErrors::CException, std::bad_alloc)
 {
-#ifndef NDEBUG
-    std::cerr << "CRTAdvancedDriver::GetScanResult() Standard scan"
-              << std::endl;
-#endif // NDEBUG
-    CWE17Driver::GetScanResult (CellVec);
-#ifndef NDEBUG
-    std::cerr << "CRTAdvancedDriver::GetScanResult() Standard scan success, "
-              << CellVec.size() << " cells." << std::endl;
-#endif // NDEBUG
-    try
-    {
-        std::vector<CCell> RalinkCellVec;
 #ifndef NDEBUG
     std::cerr << "CRTAdvancedDriver::GetScanResult() Ralink scan"
               << std::endl;
 #endif // NDEBUG
-        CRTDriver::GetScanResult (m_PrivateIoctl, MagicNumber, RalinkCellVec);
+    CRTDriver::GetScanResult (m_PrivateIoctl, MagicNumber, CellVec);
 #ifndef NDEBUG
     std::cerr << "CRTAdvancedDriver::GetScanResult() Ralink scan success, "
-              << RalinkCellVec.size() << " cells." << std::endl;
+              << CellVec.size() << " cells." << std::endl;
 #endif // NDEBUG
-        for (unsigned i (0) ; i < RalinkCellVec.size() && i < CellVec.size() ;
-                                                                            ++i)
-            if (RalinkCellVec [i].GetEncryptionD().GetAuth() == WPAPSK ||
-                RalinkCellVec [i].GetEncryptionD().GetAuth() == WPANONE)
-                    // Both results seem to be always in the same order.
-                CellVec [i].GetEncryptionD() =
-                                            RalinkCellVec [i].GetEncryptionD();
-    }
-    catch (const nsErrors::CException& Exc)
-    {
-#ifndef NDEBUG
-        std::cerr << "CRTAdvancedDriver::GetScanResult() Ralink scan failed : "
-                  << Exc.GetMsg() << ' ' << Exc.GetCode() << std::endl;
-#endif // NDEBUG
-    } // We have some results anyway.
 
 } // GetScanResult()
 
@@ -623,6 +873,7 @@ nsWireless::CRTAdvancedDriver::CRTAdvancedDriver (const std::string& DeviceName,
       m_SetIoctl (InvalidIoctl), m_RfmontxIoctl (InvalidIoctl), m_Flags (0),
       m_BGProtection (AutoProtection), m_TxPreamble (AutoPreamble)
 {
+    int GetRfMontxIoctl (InvalidIoctl);
     try
     {
         const unsigned TabArgsSize (16);
@@ -633,12 +884,7 @@ nsWireless::CRTAdvancedDriver::CRTAdvancedDriver (const std::string& DeviceName,
             else if (!strcmp (TabArgs [i].name, "rfmontx"))
                 m_RfmontxIoctl = TabArgs [i].cmd;
             else if (!strcmp (TabArgs [i].name, "get_rfmontx"))
-            {
-                int* const pResult (reinterpret_cast<int*> (m_Data.name));
-                *pResult = 0;
-                Ioctl (TabArgs [i].cmd, "Can't get rfmontx status.");
-                if (*pResult) m_Flags |= Rfmontx;
-            }
+                GetRfMontxIoctl = TabArgs [i].cmd;
             else
                 (*pHandler) (TabArgs [i]);
     }
@@ -652,30 +898,43 @@ nsWireless::CRTAdvancedDriver::CRTAdvancedDriver (const std::string& DeviceName,
     if (m_SetIoctl == InvalidIoctl)
         throw nsErrors::CException ("Can't find \"set\" private ioctl.",
                                     nsErrors::RTAdvancedSetIoctlNotFound);
-        // Get STA config :
-    SPrivateConfig PrivateConfig;
-    m_Data.data.pointer = reinterpret_cast< ::caddr_t> (&PrivateConfig);
-    m_Data.data.length = sizeof (PrivateConfig);
-    m_Data.data.flags = StaConfigMagicNumber;
-    Ioctl (m_PrivateIoctl, "Can't get STA config through special ioctl.");
-    if (PrivateConfig.EnableTxBurst) m_Flags |= TxBurst;
-    if (PrivateConfig.EnableTurboRate) m_Flags |= TurboRate;
-    m_BGProtection = BGProtection_e (PrivateConfig.UseBGProtection);
-    if (PrivateConfig.UseOfdmRatesIn11gAdhoc) m_Flags |= AdHocOFDM;
+    try
+    {
+        if (GetRfMontxIoctl != InvalidIoctl)
+        {
+            int* const pResult (reinterpret_cast<int*> (m_Data.name));
+            *pResult = 0;
+            Ioctl (GetRfMontxIoctl, "Can't get rfmontx status.");
+            if (*pResult) m_Flags |= Rfmontx;
+        }
 
-        // Get TxPreamble :
-    m_Data.data.pointer = reinterpret_cast< ::caddr_t> (&m_TxPreamble);
-    m_Data.data.length = sizeof (m_TxPreamble);
-    m_Data.data.flags = PreambleMagicNumber;
-    Ioctl (m_PrivateIoctl, "Can't get Tx Preamble through special ioctl.");
+        SPrivateConfig PrivateConfig;
+        m_Data.data.pointer = reinterpret_cast< ::caddr_t> (&PrivateConfig);
+        m_Data.data.length = sizeof (PrivateConfig);
+        m_Data.data.flags = StaConfigMagicNumber;
+        Ioctl (m_PrivateIoctl, "Can't get STA config through special ioctl.");
+        if (PrivateConfig.EnableTxBurst) m_Flags |= TxBurst;
+        if (PrivateConfig.EnableTurboRate) m_Flags |= TurboRate;
+        m_BGProtection = BGProtection_e (PrivateConfig.UseBGProtection);
+        if (PrivateConfig.UseOfdmRatesIn11gAdhoc) m_Flags |= AdHocOFDM;
 
-        // Get wireless mode :
-    DriverWirelessMode_e Mode;
-    m_Data.data.pointer = reinterpret_cast< ::caddr_t> (&Mode);
-    m_Data.data.length = sizeof (Mode);
-    m_Data.data.flags = PhyModeMagicNumber;
-    Ioctl (m_PrivateIoctl, "Cant get wireless mode through special ioctl.");
-    if (Mode == B) m_Flags |= IsB_Only;
+        m_Data.data.pointer = reinterpret_cast< ::caddr_t> (&m_TxPreamble);
+        m_Data.data.length = sizeof (m_TxPreamble);
+        m_Data.data.flags = PreambleMagicNumber;
+        Ioctl (m_PrivateIoctl, "Can't get Tx Preamble through special ioctl.");
+
+        DriverWirelessMode_e Mode;
+        m_Data.data.pointer = reinterpret_cast< ::caddr_t> (&Mode);
+        m_Data.data.length = sizeof (Mode);
+        m_Data.data.flags = PhyModeMagicNumber;
+        Ioctl (m_PrivateIoctl, "Cant get wireless mode through special ioctl.");
+        if (Mode == B) m_Flags |= IsB_Only;
+    }
+    catch (const nsErrors::CSystemExc& Exc)
+    {
+        if (Exc.GetCode() != ENETDOWN)
+            throw;
+    }
 
 } // CRTAdvancedDriver()
 
@@ -1041,7 +1300,7 @@ void nsWireless::CRT73Driver::SetRfmontx (bool B) throw (nsErrors::CSystemExc)
 {
     char Buffer (B ? '1' : '0');
     m_Data.data.pointer = reinterpret_cast< ::caddr_t> (&Buffer);
-    m_Data.data.length = 2;
+    m_Data.data.length = sizeof (Buffer);
     m_Data.data.flags = 0;
     Ioctl (GetRfmontxIoctl(), "Can't set injection mode.");
     SetFlag (Rfmontx, B);
