@@ -41,14 +41,6 @@ namespace
     } // LabelWidget()
 
 
-    const unsigned TabAuthRT2400Size (2);
-    const nsWireless::AuthType_e TabAuthRT2400 [TabAuthRT2400Size] =
-                                        {nsWireless::Open, nsWireless::Shared};
-
-    const unsigned TabAuthRT2500Size (3);
-    const nsWireless::AuthType_e TabAuthRT2500 [TabAuthRT2500Size] =
-                    {nsWireless::Open, nsWireless::Shared, nsWireless::WPAPSK};
-
     const unsigned TabAuthAdHocRTxxSize (3);
     const nsWireless::AuthType_e TabAuthAdHocRTxx [TabAuthAdHocRTxxSize] =
                 {nsWireless::Open, nsWireless::Shared, nsWireless::WPANONE};
@@ -65,6 +57,8 @@ namespace
     const nsWireless::EncryptType_e TabWPAPSKEncrypt [TabWPAPSKEncryptSize] =
                                             {nsWireless::TKIP, nsWireless::AES};
 
+    const unsigned TabAuthRT2500Size (3);
+    const unsigned TabAuthRT2400Size (2);
     const unsigned TabAuthRTxxSize (4);
     const nsWireless::AuthType_e TabAuthRTxx [TabAuthRTxxSize] =
                                     {nsWireless::Open, nsWireless::Shared,
@@ -99,6 +93,27 @@ namespace
 
     } // ApplyRTxxProfile()
 
+
+    void DisplayDriverLimitations (const nsWireless::AuthType_e* Auths,
+                        unsigned NAuth, const nsWireless::CCell& Cell,
+                                                ::GtkWindow* pWindow) throw()
+    {
+        const nsWireless::AuthType_e DesiredAuth
+                                            (Cell.GetEncryptionD().GetAuth());
+        while (NAuth)
+            if (Auths[--NAuth] == DesiredAuth)
+                return;
+        ::GtkWidget* const pDialog (::gtk_message_dialog_new (pWindow,
+                            GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
+                            GTK_BUTTONS_OK, "The device cannot be configured "
+                            " for this network because %s is unsupported, this"
+                            " could be a driver limitation .",
+                            nsWireless::GetAuthName (DesiredAuth).c_str()));
+        ::gtk_dialog_run (GTK_DIALOG (pDialog));
+        ::gtk_widget_destroy (pDialog);
+
+    } // DisplayDriverLimitations()
+
 } // anonymous namespace
 
 
@@ -117,14 +132,14 @@ const char* const nsCore::CRT2570Module::NewProfileDefaultName ("New profile");
 void nsCore::CRT2400CLIModule::ApplyProfile (unsigned Pos)
                                                     throw (nsErrors::CSystemExc)
 {
-    ApplyRT2400Profile (m_ProfilesFile [Pos], m_Driver);
+    ApplyRT2400Profile (m_ProfilesFile.GetProfile (Pos), m_Driver);
 
 } // ApplyProfile()
 
 
 nsGUI::CRT2500ConfigDialog::CRT2500ConfigDialog () throw()
     : CSharedConfigDialog (TabModeRTxx, TabModeRTxxSize,
-                           TabAuthRT2500, TabAuthRT2500Size,
+                           TabAuthRTxx, TabAuthRT2500Size,
                            TabNoneEncrypt, TabNoneEncryptSize,
                            TabSharedEncrypt, TabSharedEncryptSize,
                            TabWPAPSKEncrypt, TabWPAPSKEncryptSize)
@@ -146,8 +161,8 @@ void nsGUI::CRT2500ConfigDialog::ModeChangedCallBack (::GtkToggleButton*,
     }
     else
     {
-        This->GetAuthBox().SetElem (TabAuthRT2500, TabAuthRT2500Size);
-        This->GetAuthBox().SetActive (TabAuthRT2500 [0]);
+        This->GetAuthBox().SetElem (TabAuthRTxx, TabAuthRT2500Size);
+        This->GetAuthBox().SetActive (TabAuthRTxx [0]);
     }
 
 } // ModeChangedCallBack()
@@ -191,17 +206,27 @@ nsCore::CRT2400Module::CRT2400Module (const std::string& DeviceName,
       m_ProfileEditor (new nsGUI::CSharedConfigDialog
                                       (TabModeRTxx, TabModeRTxxSize,
                                        m_pDriver->GetSupportedFreq(),
-                                       TabAuthRT2400, TabAuthRT2400Size,
+                                       TabAuthRTxx, TabAuthRT2400Size,
                                        TabNoneEncrypt, TabNoneEncryptSize,
                                        TabSharedEncrypt, TabSharedEncryptSize,
-                                       0, 0)),
-      m_ProfilesFile (Path)
+                                       0, 0))
 {
     m_ProfilesFile.Extract();
 
 } // CRT2400Module()
 
 
+nsUserData::CProfile* nsCore::CRT2400Module::NewProfile
+              (::GtkWindow* pMainWindow, const nsWireless::CCell& Cell) throw()
+{
+    DisplayDriverLimitations (TabAuthRTxx, TabAuthRT2400Size, Cell,
+                                                                pMainWindow);
+    m_ProfilesFile.NewProfile (NewProfileDefaultName, Cell);
+    return EditNewProfile (pMainWindow);
+
+} // NewProfile()
+
+
 void nsCore::CRT2400Module::BecomeRoot () throw (nsErrors::CException,
                                                  std::bad_alloc)
 {
@@ -218,40 +243,35 @@ void nsCore::CRT2400Module::RecordProfiles () throw (nsErrors::CException)
 } // RecordProfiles()
 
 
-void nsCore::CRT2400Module::Connect (const nsWireless::CCell& Cell)
-                                   throw (nsErrors::CSystemExc, std::bad_alloc)
-{
-    if (Cell.GetMode() == nsWireless::AdHoc);
-    {
-        if (Cell.GetChannel())
-            m_pDriver->SetChannel (Cell.GetChannel());
-        else throw nsErrors::CSystemExc ("Can't connect to an ad-hoc network"
-                                         " this way, please use a profile.",
-                                         nsErrors::RT2400AdHocLimitation);
-    }
-    m_pDriver->SetMode (nsWireless::GetMatchingMode (Cell.GetMode()));
-    m_pDriver->SetSSID (Cell.GetSSID());
-    m_pDriver->SetEncryption (Cell.GetEncryptionD());
-
-} // Connect()
-
-
 void nsCore::CRT2400Module::ApplyProfile (unsigned Pos)
                                                 throw (nsErrors::CSystemExc)
 {
-    ApplyRT2400Profile (m_ProfilesFile [Pos], *m_pDriver);
+    ApplyRT2400Profile (m_ProfilesFile.GetProfile (Pos), *m_pDriver);
 
 } // ApplyProfile()
 
 
-nsUserData::CProfile* nsCore::CRT2400Module::EditNewProfile () throw()
+void nsCore::CRT2400Module::ApplyProfile (unsigned Pos,
+            const nsWireless::CMacAddress& Address) throw (nsErrors::CSystemExc)
 {
-    if (!m_ProfileEditor.Edit (m_ProfilesFile.back()))
+    ApplyRT2400Profile (m_ProfilesFile.GetProfile (Pos), *m_pDriver);
+    try {m_pDriver->SetAPMacAddr (Address);}
+    catch (const nsErrors::CSystemExc&) {} // May fail.
+
+} // ApplyProfile()
+
+
+nsUserData::CProfile* nsCore::CRT2400Module::EditNewProfile
+                                             (::GtkWindow* pMainWindow) throw()
+{
+    nsUserData::CProfile* const pProfile (&m_ProfilesFile.GetProfile
+                                                (m_ProfilesFile.Size() - 1U));
+    if (!m_ProfileEditor.Edit (pMainWindow, *pProfile))
     {
         m_ProfilesFile.DeleteLastProfile();
         return 0;
     }
-    return &m_ProfilesFile.back();
+    return pProfile;
 
 } // EditNewProfile()
 
@@ -259,7 +279,7 @@ nsUserData::CProfile* nsCore::CRT2400Module::EditNewProfile () throw()
 void nsCore::CRTAdvancedCLIModule::ApplyProfile (unsigned Pos)
                                                     throw (nsErrors::CSystemExc)
 {
-    ApplyRTxxProfile (m_ProfilesFile [Pos], *m_pDriver);
+    ApplyRTxxProfile (m_ProfilesFile.GetProfile (Pos), *m_pDriver);
 
 } // ApplyProfile()
 
@@ -268,7 +288,7 @@ nsCore::CRTAdvancedModule::CRTAdvancedModule
         (nsWireless::CRTAdvancedDriver* pDriver, nsGUI::CConfigDialog* pDialog,
          const std::string& Path, nsGUI::CGtkGUI* pUI, bool AdHofdmSupport)
                                   throw (nsErrors::CException, std::bad_alloc)
-    : m_pDriver (pDriver), m_ProfileEditor (pDialog), m_ProfilesFile (Path),
+    : m_pDriver (pDriver), m_ProfileEditor (pDialog),
       m_pTable (GTK_TABLE (::gtk_table_new (5, 2, true))), m_pUI (pUI),
       m_CurrentActiveBoxItem (0), m_IsSkipped (false)
 {
@@ -307,6 +327,16 @@ nsCore::CRTAdvancedModule::CRTAdvancedModule
 } // CRTAdvancedModule()
 
 
+nsUserData::CProfile* nsCore::CRTAdvancedModule::NewProfile
+              (::GtkWindow* pMainWindow, const nsWireless::CCell& Cell) throw()
+{
+    DisplayDriverLimitations (TabAuthRTxx, TabAuthRTxxSize, Cell, pMainWindow);
+    m_ProfilesFile.NewProfile (NewProfileDefaultName, Cell);
+    return EditNewProfile (pMainWindow);
+
+} // NewProfile()
+
+
 void nsCore::CRTAdvancedModule::RecordProfiles () throw (nsErrors::CException)
 {
     m_ProfilesFile.Record();
@@ -350,39 +380,35 @@ void nsCore::CRTAdvancedModule::CreateCheckButton (
 } // CreateCheckButton()
 
 
-nsUserData::CProfile* nsCore::CRTAdvancedModule::EditNewProfile () throw()
+nsUserData::CProfile* nsCore::CRTAdvancedModule::EditNewProfile
+                                             (::GtkWindow* pMainWindow) throw()
 {
-    if (!m_ProfileEditor.Edit (m_ProfilesFile.back()))
+    nsUserData::CProfile* const pProfile (&m_ProfilesFile.GetProfile
+                                                (m_ProfilesFile.Size() - 1U));
+    if (!m_ProfileEditor.Edit (pMainWindow, *pProfile))
     {
         m_ProfilesFile.DeleteLastProfile();
         return 0;
     }
-    return &m_ProfilesFile.back();
+    return pProfile;
 
 } // EditNewProfile()
 
 
-void nsCore::CRTAdvancedModule::Connect (const nsWireless::CCell& Cell)
-                                throw (nsErrors::CSystemExc, std::bad_alloc)
-{
-    m_pDriver->SetMode (nsWireless::GetMatchingMode (Cell.GetMode()));
-    m_pDriver->SetChannel (Cell.GetChannel());
-    m_pDriver->SetSSID (Cell.GetSSID());
-    m_pDriver->SetEncryption (Cell.GetEncryptionD());
-    m_pDriver->SetSSID (Cell.GetSSID());
-    try
-    {
-        m_pDriver->SetAPMacAddr (Cell.GetAPMacAddress());
-    }
-    catch (...) {} // It's not useful anyway.
-
-} // Connect()
-
-
 void nsCore::CRTAdvancedModule::ApplyProfile (unsigned Pos)
                                                 throw (nsErrors::CSystemExc)
 {
-    ApplyRTxxProfile (m_ProfilesFile [Pos], *m_pDriver);
+    ApplyRTxxProfile (m_ProfilesFile.GetProfile (Pos), *m_pDriver);
+
+} // ApplyProfile()
+
+
+void nsCore::CRTAdvancedModule::ApplyProfile (unsigned Pos,
+            const nsWireless::CMacAddress& Address) throw (nsErrors::CSystemExc)
+{
+    ApplyRTxxProfile (m_ProfilesFile.GetProfile (Pos), *m_pDriver);
+    try {m_pDriver->SetAPMacAddr (Address);}
+    catch (const nsErrors::CSystemExc&) {} // May fail.
 
 } // ApplyProfile()
 
@@ -477,7 +503,7 @@ void nsCore::CRTAdvancedModule::CommonCButtonCallBack
 void nsCore::CRT2570CLIModule::ApplyProfile (unsigned Pos)
                                                     throw (nsErrors::CSystemExc)
 {
-    ApplyRTxxProfile (m_ProfilesFile [Pos], m_Driver);
+    ApplyRTxxProfile (m_ProfilesFile.GetProfile (Pos), m_Driver);
 
 } // ApplyProfile()
 
@@ -487,7 +513,7 @@ nsCore::CRT2570Module::CRT2570Module (const std::string& DeviceName,
                                       nsGUI::CGtkGUI* pUI)
                                 throw (nsErrors::CException, std::bad_alloc)
     : m_pDriver (new nsWireless::CRT2570Driver (DeviceName)),
-      m_ProfileEditor (new nsGUI::CRTxxConfigDialog()), m_ProfilesFile (Path),
+      m_ProfileEditor (new nsGUI::CRTxxConfigDialog()),
       m_pUI (pUI), m_pBox (GTK_BOX (::gtk_vbox_new (true, 2))),
       m_CurrentActiveBoxItem (0), m_IsSkipped (false)
 {
@@ -499,6 +525,16 @@ nsCore::CRT2570Module::CRT2570Module (const std::string& DeviceName,
 } // CRT2570Module()
 
 
+nsUserData::CProfile* nsCore::CRT2570Module::NewProfile
+              (::GtkWindow* pMainWindow, const nsWireless::CCell& Cell) throw()
+{
+    DisplayDriverLimitations (TabAuthRTxx, TabAuthRTxxSize, Cell, pMainWindow);
+    m_ProfilesFile.NewProfile (NewProfileDefaultName, Cell);
+    return EditNewProfile (pMainWindow);
+
+} // NewProfile()
+
+
 void nsCore::CRT2570Module::RecordProfiles () throw (nsErrors::CException)
 {
     m_ProfilesFile.Record();
@@ -515,36 +551,35 @@ void nsCore::CRT2570Module::BecomeRoot () throw (nsErrors::CException,
 } // BecomeRoot()
 
 
-void nsCore::CRT2570Module::Connect (const nsWireless::CCell& Cell)
-                                    throw (nsErrors::CSystemExc, std::bad_alloc)
-{
-    m_pDriver->SetMode (nsWireless::GetMatchingMode (Cell.GetMode()));
-    m_pDriver->SetChannel (Cell.GetChannel());
-    m_pDriver->SetSSID (Cell.GetSSID());
-    m_pDriver->SetEncryption (Cell.GetEncryptionD());
-    m_pDriver->SetAPMacAddr (Cell.GetAPMacAddress());
-    m_pDriver->SetSSID (Cell.GetSSID());
-
-} // Connect()
-
-
 void nsCore::CRT2570Module::ApplyProfile (unsigned Pos)
                                                 throw (nsErrors::CSystemExc)
 {
-    ApplyRTxxProfile (m_ProfilesFile [Pos], *m_pDriver);
-    // ? FIXME ? Does the SSID need to be set again? (WPA)
+    ApplyRTxxProfile (m_ProfilesFile.GetProfile (Pos), *m_pDriver);
 
 } // ApplyProfile()
 
 
-nsUserData::CProfile* nsCore::CRT2570Module::EditNewProfile () throw()
+void nsCore::CRT2570Module::ApplyProfile (unsigned Pos,
+            const nsWireless::CMacAddress& Address) throw (nsErrors::CSystemExc)
 {
-    if (!m_ProfileEditor.Edit (m_ProfilesFile.back()))
+    ApplyRTxxProfile (m_ProfilesFile.GetProfile (Pos), *m_pDriver);
+    try {m_pDriver->SetAPMacAddr (Address);}
+    catch (const nsErrors::CSystemExc&) {} // May fail.
+
+} // ApplyProfile()
+
+
+nsUserData::CProfile* nsCore::CRT2570Module::EditNewProfile
+                                             (::GtkWindow* pMainWindow) throw()
+{
+    nsUserData::CProfile* const pProfile (&m_ProfilesFile.GetProfile
+                                                (m_ProfilesFile.Size() - 1U));
+    if (!m_ProfileEditor.Edit (pMainWindow, *pProfile))
     {
         m_ProfilesFile.DeleteLastProfile();
         return 0;
     }
-    return &m_ProfilesFile.back();
+    return pProfile;
 
 } // EditNewProfile()
 
@@ -656,6 +691,17 @@ nsCore::CRT2500Module::CRT2500Module (const std::string& DeviceName,
 } // CRT2500Module()
 
 
+nsUserData::CProfile* nsCore::CRT2500Module::NewProfile
+              (::GtkWindow* pMainWindow, const nsWireless::CCell& Cell) throw()
+{
+    DisplayDriverLimitations (TabAuthRTxx, TabAuthRT2500Size, Cell,
+                                                                pMainWindow);
+    GetProfilesFile().NewProfile (NewProfileDefaultName, Cell);
+    return EditNewProfile (pMainWindow);
+
+} // NewProfile()
+
+
 nsCore::CRT61Module::CRT61Module (const std::string& DeviceName,
                             const std::string& ProfilePath, nsGUI::CGtkGUI* pUI)
                                     throw (nsErrors::CException, std::bad_alloc)
