@@ -120,9 +120,7 @@ namespace
         std::memset(&addr, 0, sizeof (addr));
         const size_t UNIX_PATH_MAX (108U);
         addr.sun_family = AF_UNIX;
-        addr_name.copy (addr.sun_path + 1, UNIX_PATH_MAX - 2);
-        addr.sun_path [addr_name.size() > UNIX_PATH_MAX - 1 ? UNIX_PATH_MAX
-                                                    : addr_name.size()] = '\0';
+        addr_name.copy (addr.sun_path + 1, UNIX_PATH_MAX - 1);
 
     } // InitSockaddrUn()
 
