@@ -29,6 +29,10 @@
 #include <iostream>
 #endif // NDEBUG
 
+extern "C"{
+#include <net/if_arp.h>
+}
+
 #include "WE17Driver.h"
 #include "ErrorsCode.h"
 
@@ -110,11 +114,15 @@ nsWireless::CWE17Driver::CWE17Driver (const std::string& Name)
         std::cerr << "CWE17Driver::CWE17Driver() Can't get interface"
                      " information : " << Exc.GetCode() << std::endl;
 #endif // NDEBUG
-        if (Exc.GetCode() != EOPNOTSUPP) throw;
+        if (Exc.GetCode() != EOPNOTSUPP && Exc.GetCode() != ENETDOWN) throw;
     }
     if (m_SupportedFreqVec.empty())
     {
-        for (unsigned i (1) ; i < 14 ; ++i) // I put the whole spectrum.
+#ifndef NDEBUG
+        std::cerr << "CWE17Driver::CWE17Driver() generate channel list."
+                  << std::endl;
+#endif // NDEBUG
+        for (unsigned i (1) ; i < 14 ; ++i) // Let's put the whole spectrum.
             m_SupportedFreqVec.push_back (CFreq (i, 2.412 + (i - 1) * 0.005));
         m_SupportedFreqVec.push_back (CFreq (14, 2.484));
     }
@@ -391,8 +399,12 @@ nsWireless::CMacAddress nsWireless::CWE17Driver::GetAPMacAddr () const
         throw;
     }
 #ifndef NDEBUG
-    std::cerr << "CWE17Driver::GetAPMacAddr() "
-              << std::string (m_Req.u.ap_addr.sa_data, 0, 14) << std::endl;
+    std::cerr << "CWE17Driver::GetAPMacAddr() " << std::hex;
+    const unsigned char* const RawData (reinterpret_cast<const unsigned char*>
+                                                    (m_Req.u.ap_addr.sa_data));
+    for (unsigned i (0) ; i < 6 ; ++i)
+        std::cerr << unsigned (RawData [i]) << ' ';
+    std::cerr << std::dec << std::endl;
 #endif // NDEBUG
     return CMacAddress (m_Req.u.ap_addr.sa_data);
 
@@ -405,7 +417,8 @@ void nsWireless::CWE17Driver::SetAPMacAddr (const CMacAddress& Addr)
 #ifndef NDEBUG
     std::cerr << "CWE17Driver::SetAPMacAddr() " << Addr.GetStr() << std::endl;
 #endif // NDEBUG
-    strncpy (m_Req.u.ap_addr.sa_data, Addr.Get(), CMacAddress::Size);
+    m_Req.u.ap_addr.sa_family = ARPHRD_ETHER;
+    memcpy (m_Req.u.ap_addr.sa_data, Addr.Get(), CMacAddress::Size);
     Ioctl (SIOCSIWAP, std::string ("Can't set access point Mac address : ") +
                       Addr.GetStr());
 
@@ -434,13 +447,11 @@ void nsWireless::CWE17Driver::GetScanResult (std::vector<CCell>& CellVec) const
     std::cerr << "CWE17Driver::GetScanResult()" << std::endl;
 #endif // NDEBUG
     m_Req.u.data.flags = 0;
-    unsigned BufferSize (256); // In iwlist, the starting size is 128.
-    unsigned char* Buffer;
+    std::vector<char> Buffer (256); // In iwlist, the starting size is 128.
     for ( ; ; )
     {
-        Buffer = new unsigned char [BufferSize];
-        m_Req.u.data.pointer = reinterpret_cast < ::caddr_t> (Buffer);
-        m_Req.u.data.length = BufferSize;
+        m_Req.u.data.pointer = reinterpret_cast < ::caddr_t> (&Buffer [0]);
+        m_Req.u.data.length = Buffer.size();
         try
         {
             Ioctl (SIOCGIWSCAN, "Can't get scan results.");
@@ -449,36 +460,28 @@ void nsWireless::CWE17Driver::GetScanResult (std::vector<CCell>& CellVec) const
         catch (const nsErrors::CSystemExc& Exc)
         {
 #ifndef NDEBUG
-            std::cerr << "\n\tBuffer too small" << std::endl;
+            std::cerr << "\n\tCWE17Driver::GetScanResult() failed" << std::endl;
 #endif // NDEBUG
             if (Exc.GetCode() == E2BIG)
             {       // If the driver gives the expected size.
-                if (m_Req.u.data.length > BufferSize)
-                    BufferSize = m_Req.u.data.length;
-                    // Else we try a bigger size.
-                else BufferSize *= 2;
+#ifndef NDEBUG
+                std::cerr << " : Buffer too small" << std::endl;
+#endif // NDEBUG
+                if (m_Req.u.data.length > Buffer.size())
+                    Buffer.resize (m_Req.u.data.length);
+                    // Else double the size.
+                else
+                    Buffer.resize (Buffer.size() * 2U);
             }
             else
-            {
-                delete[] Buffer;
                 throw;
-            }
         }
-        delete[] Buffer;
     }
         // Things are MUCH more complicated in iwlib.c...
-    try
-    {
-        for (unsigned Offset (0) ; Offset < m_Req.u.data.length ; )
-            Offset += FillCellVec (reinterpret_cast<const ::iw_event*> 
-                                                (Buffer + Offset), CellVec);
-    }
-    catch (...)
-    {
-        delete[] Buffer;
-        throw;
-    }
-    delete[] Buffer;
+    const char* const pEvents (&Buffer [0]);
+    for (unsigned Offset (0) ; Offset < m_Req.u.data.length ; )
+        Offset += FillCellVec (reinterpret_cast<const ::iw_event*> 
+                                                (pEvents + Offset), CellVec);
 
 } // GetScanResult()
 
@@ -488,17 +491,24 @@ unsigned nsWireless::CWE17Driver::FillCellVec (const ::iw_event* pEvent,
                                                 throw (nsErrors::CException)
 {
 #ifndef NDEBUG
-    std::cerr << "\n\tEvent : " << pEvent->cmd;
+    std::cerr << "\n\tEvent : " << std::hex << pEvent->cmd << std::dec;
 #endif // NDEBUG
-        // Jean Tourrilhes in iwlib.c warn that data may be unaligned.
+        // Jean Tourrilhes in iwlib.c warns that data may be unaligned.
     switch (pEvent->cmd)
     {
       case SIOCGIWAP :
+      {
 #ifndef NDEBUG
-        std::cerr << "\n\tSIOCGIWAP : "
-                  << std::string (pEvent->u.ap_addr.sa_data, 0, 14);
+        std::cerr << "\n\tSIOCGIWAP : " << std::hex;
+        const unsigned char* const RawData
+                                        (reinterpret_cast<const unsigned char*>
+                                                    (m_Req.u.ap_addr.sa_data));
+        for (unsigned i (0) ; i < 6 ; ++i)
+            std::cerr << unsigned (RawData [i]) << ' ';
+        std::cerr << std::dec << std::endl;
 #endif // NDEBUG
         Vec.push_back (CCell (CMacAddress (pEvent->u.ap_addr.sa_data)));
+      }
       break;
 
       case SIOCGIWMODE :
@@ -573,7 +583,7 @@ unsigned nsWireless::CWE17Driver::FillCellVec (const ::iw_event* pEvent,
 #ifndef NDEBUG
         std::cerr << "\n\tSIOCGIWFREQ : " << pEvent->u.freq.m << ' '
                   << pEvent->u.freq.e << ' ' << pEvent->u.freq.i << ' '
-                  << pEvent->u.freq.flags;
+                  << pEvent->u.freq.flags << std::endl;
 #endif // NDEBUG
         if (pEvent->u.freq.m > 1000)
             Vec.back().m_Channel = GetMatchingFreq (0,
@@ -598,7 +608,7 @@ unsigned nsWireless::CWE17Driver::FillCellVec (const ::iw_event* pEvent,
 
 #ifndef NDEBUG
       default :
-        std::cerr << "Unknown.";
+        std::cerr << " Unknown.";
       //break;
 #endif // NDEBUG
     }
@@ -674,7 +684,7 @@ void nsWireless::CWE17Driver::Commit () throw (nsErrors::CSystemExc)
 #ifndef NDEBUG
     std::cerr << "CWE17Driver::Commit()\n";
 #endif // NDEBUG
-    m_Data.data.pointer = reinterpret_cast< ::caddr_t> (0);
+    m_Data.data.pointer = 0;
     m_Data.data.length = 0;
     m_Data.data.flags = 0;
     Ioctl (SIOCSIWCOMMIT, "Can't commit.");
@@ -696,5 +706,8 @@ void nsWireless::CWE17Driver::GetSupportedRates (std::vector<int>& RatesVec)
     for (unsigned i (0) ; i < Range.num_bitrates ; ++i)
         RatesVec.push_back (Range.bitrate [i] / TxUnit);
     std::sort (RatesVec.rbegin(), RatesVec.rend());
+#ifndef NDEBUG
+    std::cerr << "Exit CWE17Driver::GetSupportedRates()\n";
+#endif // NDEBUG
 
 } // GetSupportedRates()
