@@ -68,7 +68,7 @@ void nsGUI::CKeyEntry::NewTextTyped  (::GtkEditable* pEditable, char* NewText,
                                       CKeyEntry* This) throw()
 {
     if (!::gtk_toggle_button_get_active (This->m_pIsASCIIButton) &&
-                                                    !std::isxdigit (*NewText))
+	!std::isxdigit (static_cast<unsigned char> (*NewText)))
         ::g_signal_stop_emission_by_name (pEditable, "insert-text");
 
 } // NewTextTyped()
@@ -80,7 +80,7 @@ void nsGUI::CKeyEntry::ASCIIButtonToggled (::GtkToggleButton*, CKeyEntry* This)
     if (!::gtk_toggle_button_get_active (This->m_pIsASCIIButton))
         for (const char* KeyStr (::gtk_entry_get_text (This->m_pEntry)) ;
                                                             *KeyStr ; ++KeyStr)
-            if (!std::isxdigit (*KeyStr))
+            if (!std::isxdigit (static_cast<unsigned char> (*KeyStr)))
             {
                 This->Clear();
                 break;
@@ -139,7 +139,7 @@ nsGUI::CConfigDialog::CConfigDialog (const nsWireless::Mode_e* ModeTab,
                                               GTK_STOCK_OK, GTK_RESPONSE_OK,
                                               GTK_STOCK_CANCEL,
                                               GTK_RESPONSE_CANCEL,
-                                              reinterpret_cast<void*> (0)))),
+                                              static_cast<void*> (0)))),
       m_pNameEntry (GTK_ENTRY (::gtk_entry_new())),
       m_pSSIDEntry (GTK_ENTRY (::gtk_entry_new())),
       m_ModeBox (ModeTab, TabModeSize),
@@ -169,11 +169,9 @@ nsGUI::CConfigDialog::CConfigDialog (const nsWireless::Mode_e* ModeTab,
                                                         throw (std::bad_alloc)
     : m_ChannelTab (0), m_ChannelTabSize (0),
       m_pDialog (GTK_DIALOG (::gtk_dialog_new_with_buttons
-                                             (0, 0,  ::GtkDialogFlags (0),
-                                              GTK_STOCK_OK, GTK_RESPONSE_OK,
-                                              GTK_STOCK_CANCEL,
-                                              GTK_RESPONSE_CANCEL,
-                                              reinterpret_cast<void*> (0)))),
+                             (0, 0,  ::GtkDialogFlags (0), GTK_STOCK_OK,
+                                              GTK_RESPONSE_OK, GTK_STOCK_CANCEL,
+                              GTK_RESPONSE_CANCEL, static_cast<void*> (0)))),
       m_pNameEntry (GTK_ENTRY (::gtk_entry_new())),
       m_pSSIDEntry (GTK_ENTRY (::gtk_entry_new())),
       m_ModeBox (ModeTab, TabModeSize),
